#!/bin/ksh

#**************************************************************
# Author: Sbastien Denvil
# Contact: sebastien.denvil.@ipsl.jussieu.fr
# $Date$
# $Author$
# $Revision$
# IPSL (2006)
#  This software is governed by the CeCILL licence see libIGCM/libIGCM_CeCILL.LIC
# History:
# Modification:
#
#**************************************************************

#=========================================================
# The documentation of this file can be automatically generated
# if you use the prefix #D- for comments to be extracted. 
# Extract with command: cat lib* | grep "^#D-" | cut -c "4-"
#=========================================================

#D-#=================================================
#D-LibIGCM_sys for Brodie
#D-#=================================================
#D-
#D- This ksh library if a layer under some usefull 
#D-environment variables and shell commands.
#D-All those definitions depend on host particularities.
#D-It manages a stack mechanism and test validity of operations.
#D-All function described bellow must be prefixed by IGCM_sys.

#====================================================
# libIGCM_sys PARAMETERS
#====================================================

#====================================================
# set DEBUG_sys to true to output calls of function
typeset -r DEBUG_sys=${DEBUG_sys:=true}

#====================================================
# Turn in dry run mode ? (sys_Put_Rest, sys_Put_Out, sys_Get)
typeset -r DRYRUN=${DRYRUN:=0}

# YOU MUST COMPILE YOUR EXE FILES FOR DRYRUN MODE !
# -------------------------------------------------------------------------------------
# | DRYRUN=  |  Date computations, | sys_Get    |  Exe    | sys_Put_Out; sys_Put_Rest |
# |          |  Cp/Exe param files |            |  Chmod  |                           |
# |          |      Qsub           |            |         |                           |
# -------------------------------------------------------------------------------------
# |    0     |       yes           |    yes     |  yes    |      yes                  |
# -------------------------------------------------------------------------------------
# |    1     |       yes           |    yes     |  yes    |      no                   |
# -------------------------------------------------------------------------------------
# |    2     |       yes           |    yes     |  no     |      no                   |
# -------------------------------------------------------------------------------------
# |    3     |       yes           |    no      |  no     |      no                   |
# -------------------------------------------------------------------------------------

#=====================================================
# Global Variables :
#=====================================================
# Language : "fr" or "en"
typeset -r MYLANG="fr"

#=====================================================
# Host and user names 
# $hostname ou hostname 
typeset -r HOST=$( hostname )
# $username ou whoami
typeset -r LOGIN=$( whoami )

#D-
#D-#==================================================
#D-Program used in libIGCM
#D-#==================================================

# rsync with path
typeset -r RSYNC=/home/rech/ces/rces452/RSYNC_SX8/bin/rsync
# RSYNC_opt args to "local rsync"
typeset -r RSYNC_FRONT="rsh rhodes /usr/bin/rsync "
# RSYNC_opt args to rsync
typeset -r RSYNC_opt="-va"
# RSYNC_opt args to "remote rsync"
# ie storage filesystem
typeset -r RHOST=gaya.idris.fr
typeset -r REMOTE_RSYNC=/u/rech/ces/rces452/RSYNC/bin/rsync

#====================================================
# Host specific DIRECTORIES
#====================================================

#====================================================
#- R_EXE   (==> BIN_DIR = ${MODIPSL}/bin )
typeset -r R_EXE="${MODIPSL}/bin"

#====================================================
#- SUBMIT_DIR : submission dir
typeset -r SUBMIT_DIR=${SUBMIT_DIR:=${PBS_O_WORKDIR}}

#====================================================
#- ARCHIVE
typeset -r ARCHIVE=$( rsh gaya -n env | grep PWD | awk "-F=" '{print $2}' )

#====================================================
#- Mirror libIGCM from brodie to rhodes 
typeset -r MirrorlibIGCM=${MirrorlibIGCM:=true}

#====================================================
#- IN
typeset -r R_IN=${R_IN:=/u/rech/psl/rpsl035/IGCM}

#====================================================
#- OUT
typeset -r R_OUT=${ARCHIVE}/IGCM_OUT

#====================================================
#- OUT_POST 
typeset -r R_OUT_POST=$( rsh rhodes -n echo $WORKDIR | awk "-F=" "{print $2}" )/IGCM_OUT 

#====================================================
#- RUN_DIR_PATH : Temporary working directory (=> TMP)
typeset -r RUN_DIR_PATH=${RUN_DIR_PATH:=${TMPDIR}}

#====================================================
#- HOST_MPIRUN_COMMAND
typeset -r HOST_MPIRUN_COMMAND=${HOST_MPIRUN_COMMAND:="mpirun"}

#D-#==================================================
#D-function IGCM_sys_RshArchive
#D-* Purpose: Archive rsh command
#D-* Examples:
#D-
function IGCM_sys_RshArchive {
    IGCM_debug_PushStack "IGCM_sys_RshArchive" $@
    rsh gaya exec /bin/ksh <<-EOF
    ${@}
EOF
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_RshArchive : erreur."
	IGCM_debug_Exit "IGCM_sys_RshArchive"
    fi
    IGCM_debug_PopStack "IGCM_sys_RshArchive"
}

#D-#==================================================
#D-function IGCM_sys_RshPost
#D-* Purpose: Post-process rsh command
#D-* Examples:
#D-
function IGCM_sys_RshPost {
    IGCM_debug_PushStack "IGCM_sys_RshPost" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_RshPost :" $@
    fi
    rsh rhodes exec /bin/ksh ${@}
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_RshPost : erreur."
	IGCM_debug_Exit "IGCM_sys_RshPost"
    fi
    IGCM_debug_PopStack "IGCM_sys_RshPost"
}

#D-#==================================================
#D-function IGCM_sys_Mkdir
#D-* Purpose: Master locale mkdir command
#D-* Examples:
#D-
function IGCM_sys_Mkdir {
    IGCM_debug_PushStack "IGCM_sys_Mkdir" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Mkdir :" $@
    fi
    if [ ! -d ${1} ]; then
	\mkdir -p $1
	if [ $? -gt 0 ] ; then
	    echo "IGCM_sys_Mkdir : erreur."
	    IGCM_debug_Exit "IGCM_sys_Mkdir"
	fi
    fi
    # vrification :
    if [ ! -d ${1} ] ; then
	echo "IGCM_sys_Mkdir : erreur."
	IGCM_debug_Exit "IGCM_sys_Mkdir"
    fi
    IGCM_debug_PopStack "IGCM_sys_Mkdir"
}

#D-#==================================================
#D-function IGCM_sys_MkdirArchive
#D-* Purpose: Mkdir on Archive
#D-* Examples:
#D-
function IGCM_sys_MkdirArchive {
    IGCM_debug_PushStack "IGCM_sys_MkdirArchive" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_MkdirArchive :" $@
    fi
    #- creation de repertoire sur le serveur fichier
    rsh gaya -n mkdir -p $1

    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_MkdirArchive : erreur."
	IGCM_debug_Exit "IGCM_sys_MkdirArchive"
    fi
    IGCM_debug_PopStack "IGCM_sys_MkdirArchive"
}

#D-#==================================================
#D-function IGCM_sys_MkdirWork
#D-* Purpose: Mkdir on Work
#D-* Examples:
#D-
function IGCM_sys_MkdirWork {
    IGCM_debug_PushStack "IGCM_sys_MkdirWork" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_MkdirWork :" $@
    fi
    #- creation de repertoire sur le serveur fichier
    if [ ! -d ${1} ]; then 
	\mkdir -p $1
	if [ $? -gt 0 ] ; then
	    echo "IGCM_sys_MkdirWork : erreur."
	    IGCM_debug_Exit "IGCM_sys_MkdirWork"
	fi
    fi
    # vrification ?? :
    if [ ! -d ${1} ] ; then
	echo "IGCM_sys_MkdirWork : erreur."
	IGCM_debug_Exit "IGCM_sys_MkdirWork"
    fi
    IGCM_debug_PopStack "IGCM_sys_MkdirWork"
}
#IGCM_sys_MkdirWork ${RUN_DIR_PATH}
#echo "RUN_DIR_PATH ${RUN_DIR_PATH} ok."

#D-#==================================================
#D-function IGCM_sys_Cd
#D-* Purpose: master cd command
#D-* Examples:
#D-
function IGCM_sys_Cd {
    IGCM_debug_PushStack "IGCM_sys_Cd" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Cd :" $@
    fi
    \cd $1
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_Cd : erreur."
	IGCM_debug_Exit "IGCM_sys_Cd"
    fi
    IGCM_debug_PopStack "IGCM_sys_Cd"
}

#D-#==================================================
#D-function IGCM_sys_Chmod
#D-* Purpose: Chmod
#D-* Examples:
#D-
function IGCM_sys_Chmod {
    IGCM_debug_PushStack "IGCM_sys_Chmod" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Chmod :" $@
    fi
    if [ $DRYRUN -le 1 ]; then
	\chmod $@
	if [ $? -gt 0 ] ; then
	    echo "IGCM_sys_Chmod : erreur."
	    IGCM_debug_Exit "IGCM_sys_Chmod"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Chmod"
}

#D-#==================================================
#D-function IGCM_sys_FileSize
#D-* Purpose: Filesize
#D-* Examples:
#D-
function IGCM_sys_FileSize {
    IGCM_debug_PushStack "IGCM_sys_FileSize" $@

    typeset sizeF
    set +A sizeF -- $( ls -la ${1} )
    if [ $? -gt 0 ] ; then
	IGCM_debug_Exit "IGCM_sys_FileSize"
    fi
    eval ${2}=${sizeF[4]}

    IGCM_debug_PopStack "IGCM_sys_FileSize"
}

#D-#==================================================
#D-function IGCM_sys_TestDir
#D-* Purpose: Test Directory that must exists
#D-* Examples:
#D-
function IGCM_sys_TestDir {
    IGCM_debug_PushStack "IGCM_sys_TestDir" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_TestDir :" $@
    fi
    if [ ! -d ${1} ]; then
	echo "IGCM_sys_TestDir : Directory $1 does not exist."
	IGCM_debug_Exit "IGCM_sys_TestDir"
    fi
    IGCM_debug_PopStack "IGCM_sys_TestDir"
}

#D-#==================================================
#D-function IGCM_sys_TestDirArchive
#D-* Purpose: Test Directory that must exists on Archive
#D-* Examples:
#D-
function IGCM_sys_TestDirArchive {
    IGCM_debug_PushStack "IGCM_sys_TestDirArchive" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_TestDirArchive :" $@
    fi
    TEST=$( rsh gaya -n "[ -d $1 ] && echo 1 || echo 0" )
    if [ ${TEST} -ne 1 ] ; then
	echo "IGCM_sys_TestDirArchive : Directory $1 does not exist on ${ARCHIVE}."
	IGCM_debug_Exit "IGCM_sys_TestDirArchive"
    fi
    IGCM_debug_PopStack "IGCM_sys_TestDirArchive"
}

#D-#==================================================
#D-function IGCM_sys_TestFileArchive
#D-* Purpose: Test file that must NOT EXISTS on Archive
#D-* Examples:
#D-
function IGCM_sys_TestFileArchive {
    IGCM_debug_PushStack "IGCM_sys_TestFileArchive" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_TestFileArchive :" $@
    fi
    typeset RET 

    mfls ${1} | grep ${1} > /dev/null 2>&1
    RET=$?
    if [ ${RET} -gt 0 ]; then
	IGCM_debug_PopStack "IGCM_sys_TestFileArchive"
	return 1
    else
	IGCM_debug_PopStack "IGCM_sys_TestFileArchive"
	return 0
    fi
}

#D-#==================================================
#D-function IGCM_sys_Tree
#D-* Purpose: Tree directories with files on ${ARCHIVE}
#D-* Examples: IGCM_sys_Tree ${R_IN} ${R_OUT}
#D-
function IGCM_sys_Tree {
    IGCM_debug_PushStack "IGCM_sys_Tree" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Tree :" $@
    fi

    \mfls -r $@

    IGCM_debug_PopStack "IGCM_sys_Tree"
}

#D-#==================================================
#D-function IGCM_sys_Tar
#D-* Purpose: master un-tar command
#D-* Examples:
#D-
function IGCM_sys_Tar {
    IGCM_debug_PushStack "IGCM_sys_Tar" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Tar :" $@
    fi
    \tar xvf $1
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_Tar : erreur."
	IGCM_debug_Exit "IGCM_sys_Tar"
    fi
    IGCM_debug_PopStack "IGCM_sys_Tar"
}

#D-#==================================================
#D-function IGCM_sys_UnTar
#D-* Purpose: master un-tar command
#D-* Examples:
#D-
function IGCM_sys_UnTar {
    IGCM_debug_PushStack "IGCM_sys_UnTar" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_UnTar :" $@
    fi
    \tar xvf $1
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_UnTar : erreur."
	IGCM_debug_Exit "IGCM_sys_UnTar"
    fi
    IGCM_debug_PopStack "IGCM_sys_UnTar"
}

#D-#==================================================
#D-function IGCM_sys_Qsub
#D-* Purpose: Qsub new job
#D-* Examples:
#D-
function IGCM_sys_Qsub {
    IGCM_debug_PushStack "IGCM_sys_Qsub" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Qsub :" $@
    fi
    /usr/bin/nqsII/qsub -o ${Script_Output} -N ${config_UserChoices_JobName}.${CumulPeriod} < $1
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_Qsub : erreur -o ${Script_Output} -N ${config_UserChoices_JobName}.${CumulPeriod} $@."
	IGCM_debug_Exit "IGCM_sys_Qsub"
    fi
    IGCM_debug_PopStack "IGCM_sys_Qsub"
}

#D-#==================================================
#D-function IGCM_sys_QsubPost
#D-* Purpose: Qsub new job on scalaire
#D-* Examples:
#D-
function IGCM_sys_QsubPost {
    IGCM_debug_PushStack "IGCM_sys_QsubPost" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_QsubPost :" $@
    fi
    /usr/bin/nqsII/qsub -q scalaire -o ${POST_DIR}/$1.${PeriodDateEnd}.out ${libIGCM_POST}/$1.job -v ${listVarEnv}
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_QsubPost : erreur " $@
	IGCM_debug_Exit "IGCM_sys_QsubPost"
    fi
    IGCM_debug_PopStack "IGCM_sys_QsubPost"
}

#D-*************************
#D- File transfer functions 
#D-*************************
#D-

#D-#==================================================
#D-function IGCM_sys_Rsync_out
#D-* Purpose: treat return val of rsync
#D-* Examples: IGCM_sys_Rsync_out out_RET_rsync
#D-  Error values and explanations can depend on your system version.
function IGCM_sys_Rsync_out {
    RET=$1
    if [ ! $RET ] ; then
	echo "rsync error !"
    fi

    if [ $MYLANG = "fr" ]; then
	case $RET in
	    0)  return ;;
	    1)  echo "Erreur de rsync ; RERR_SYNTAX : "
		echo "Erreur de syntaxe ou d'utilisation."
		return;;
	    2)  echo "Erreur de rsync ; RERR_PROTOCOL : "
		echo "Incompatibilit de protocole."
		return;;
	    3)  echo "Erreur de rsync ; RERR_FILESELECT 3"
		echo "Erreurs  lors  de  la  slection des fichiers d'entre sortie et"
		echo "rpertoires"
		return;;
	    4)  echo "Erreur de rsync ; RERR_UNSUPPORTED"
		echo "Action demande non supporte : une tentative de manipulation de"
		echo "fichiers  64-bits  sur une plate-forme qui ne les supporte pas a"
		echo "t faite ; ou une option qui est supporte par le  client  mais"
		echo "pas par le serveur a t spcifie."
		return;;
	    10) echo "Erreur de rsync ; RERR_SOCKETIO"
		echo "Erreur dans le socket d'entre sortie"
		return;;
	    11) echo "Erreur de rsync ; RERR_FILEIO"
		echo "Erreur d'entre sortie fichier"
		return;;
	    12) echo "Erreur de rsync ; RERR_STREAMIO"
		echo "Erreur dans flux de donne du protocole rsync"
		return;;
	    13) echo "Erreur de rsync ; RERR_MESSAGEIO"
		echo "Erreur avec les diagnostics du programme"
		return;;
	    14) echo "Erreur de rsync ; RERR_IPC"
		echo "Erreur dans le code IPC"
		return;;
	    20) echo "Erreur de rsync ; RERR_SIGNAL"
		echo "SIGUSR1 ou SIGINT reu"
		return;;
	    21) echo "Erreur de rsync ; RERR_WAITCHILD"
		echo "Une erreur retourne par waitpid()"
		return;;
	    22) echo "Erreur de rsync ; RERR_MALLOC"
		echo "Erreur lors de l'allocation des tampons de mmoire de coeur"
		return;;
	    23) echo ""
                echo "Erreur fichier inexistant"
		return;;
	    30) echo "Erreur de rsync ; RERR_TIMEOUT"
		echo "Temps d'attente coul dans l'envoi/rception de donnes"
		return;;
	    *)  echo "Erreur de rsync : code de retour de rsync inconnu :" $RET
		return;;
	esac
    elif [ $MYLANG = "en" ] ; then
	case $RET in
	    0)  return;;		
	    1)  echo "rsync error : Syntax or usage error "
		return;;
	    2)  echo "rsync error : Protocol incompatibility "
		return;;
	    3)  echo "rsync error : Errors selecting input/output files, dirs"
		return;;
	    4)  echo "rsync error : Requested action not supported: an attempt"
		echo "was made to manipulate 64-bit files on a platform that cannot support"
		echo "them; or an option was specified that is supported by the client and"
		echo "not by the server."
		return;;
	    5)  echo "rsync error : Error starting client-server protocol"
		return;;
	    10) echo "rsync error : Error in socket I/O "
		return;;
	    11) echo "rsync error : Error in file I/O "
		return;;
	    12) echo "rsync error : Error in rsync protocol data stream "
		return;;
	    13) echo "rsync error : Errors with program diagnostics "
		return;;
	    14) echo "rsync error : Error in IPC code "
		return;;
	    20) echo "rsync error : Received SIGUSR1 or SIGINT "
		return;;
	    21) echo "rsync error : Some error returned by waitpid() "
		return;;
	    22) echo "rsync error : Error allocating core memory buffers "
		return;;
	    23) echo "rsync error : Partial transfer due to error"
		return;;
	    24) echo "rsync error : Partial transfer due to vanished source files"
		return;;
	    30) echo "rsync error : Timeout in data send/receive "
		return;;
	    *)  echo "rsync error : return code of rsync unknown :" $RET
		return;;
	esac
    else
	echo "unknown language $MYLANG."
	return
    fi
}
    

#D-#==================================================
#D-function IGCM_sys_Miror_libIGCM
#D-* Purpose: Mirror libIGCM PATH and lib to rhodes
#D-* Examples:
#D-
function IGCM_sys_Mirror_libIGCM {
    IGCM_debug_PushStack "IGCM_sys_Mirror_libIGCM"
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Mirror_libIGCM"
    fi

    typeset RET DEST

    IGCM_sys_RshPost <<-EOF
    mkdir -p ${PATHlibIGCM}
EOF
    rsh rhodes -n hostname > /dev/null 2>&1
    RET=$?
    if [ $RET -eq 0 ] ; then
# 	echo rcp -r ${libIGCM} rhodes:${HOME}/${PATHlibIGCM} > out_rsync 2>&1
# 	rcp -r ${libIGCM} rhodes:${HOME}/${PATHlibIGCM} >> out_rsync 2>&1
	echo ${RSYNC} ${RSYNC_opt} -e "rsh " --rsync-path=${REMOTE_RSYNC} ${libIGCM} rhodes:${HOME}/${PATHlibIGCM} > out_rsync 2>&1
	${RSYNC} ${RSYNC_opt} -e "rsh " --rsync-path=${REMOTE_RSYNC} ${libIGCM} rhodes:${HOME}/${PATHlibIGCM} >> out_rsync 2>&1
	RET=$?

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Mirror_libIGCM Warning : no libIGCM on rhodes."
	    cat out_rsync
	fi
    else
	echo "No POST-TREATMENT avaible because rhodes is down."
    fi
    IGCM_debug_PopStack "IGCM_sys_Mirror_libIGCM"
}

#====================================================
#- Call IGCM_sys_Mirror_libIGCM now !
PATHlibIGCM=$( echo ${libIGCM} | awk -F"${LOGIN}/" '{print $2}' | sed -e "s&/libIGCM&&" )
if ( $MirrorlibIGCM ) ; then
    IGCM_sys_Mirror_libIGCM
fi

#====================================================
#- libIGCM_POST
typeset -r libIGCM_POST=${HOME}/${PATHlibIGCM}/libIGCM

#D-#==================================================
#D-function IGCM_sys_Cp
#D-* Purpose: generic cp
#D-* Examples:
#D-
function IGCM_sys_Cp {
    IGCM_debug_PushStack "IGCM_sys_Cp" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Cp :" $@
    fi

    typeset RET

    echo cp $@ > out_rsync 2>&1
    \cp $@ >> out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
	echo "IGCM_sys_Cp : error."
	cat out_rsync
	IGCM_debug_Exit "IGCM_sys_Cp"
    fi
    IGCM_debug_PopStack "IGCM_sys_Cp"
}

#D-#==================================================
#D-function IGCM_sys_Mv
#D-* Purpose: generic move
#D-* Examples:
#D-
function IGCM_sys_Mv {
    IGCM_debug_PushStack "IGCM_sys_Mv" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Mv :" $@
    fi

    typeset RET

    echo mv $@ > out_rsync 2>&1
    \mv $@ >> out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
	echo "IGCM_sys_Mv : error in mv."
	cat out_rsync
	IGCM_debug_Exit "IGCM_sys_Mv"
    fi

    IGCM_debug_PopStack "IGCM_sys_Mv"
}

#D-#==================================================
#D-function IGCM_sys_Put_Dir
#D-* Purpose: Copy a complete directory on $(ARCHIVE)
#D-* Examples:
#D-
function IGCM_sys_Put_Dir {
    IGCM_debug_PushStack "IGCM_sys_Put_Dir" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Put_Dir :" $@
    fi
    if [ $DRYRUN = 0 ]; then
	if [ ! -d ${1} ] ; then
	    echo "WARNING : IGCM_sys_Put_Dir ${1} DOES NOT EXIST ."
	    IGCM_debug_PopStack "IGCM_sys_Put_Dir"
	    return
	fi

	typeset RET

	# Only if we use rsync
	#IGCM_sys_TestDirArchive $( dirname $2 )
	#
	#USUAL WAY
	rcp -r $1 gaya:$2 > out_rsync 2>&1
  	RET=$?

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Put_Dir : error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Put_Dir"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Put_Dir"
}

#D-#==================================================
#D-function IGCM_sys_Get_Dir
#D-* Purpose: Copy a complete directory from $(ARCHIVE)
#D-* Examples:
#D-
function IGCM_sys_Get_Dir {
    IGCM_debug_PushStack "IGCM_sys_Get_Dir" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Get_Dir :" $@
    fi
    if [ $DRYRUN = 0 ]; then
	if [ ! -d ${1} ] ; then
	    echo "WARNING : IGCM_sys_Get_Dir ${1} DOES NOT EXIST ."
	    IGCM_debug_PopStack "IGCM_sys_Get_Dir"
	    return
	fi

	typeset RET

	#USUAL WAY
	rcp -rp gaya:$1 $2 > out_rsync 2>&1
  	RET=$?

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Get_Dir : error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Get_Dir"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Get_Dir"
}

#D-#==================================================
#D-function IGCM_sys_Put_Rest
#D-* Purpose: Put computied restarts on $(ARCHIVE).
#D-           File and target directory must exist.
#D-* Examples:
#D-
function IGCM_sys_Put_Rest {
    IGCM_debug_PushStack "IGCM_sys_Put_Rest" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Put_Rest :" $@
    fi
    if [ $DRYRUN = 0 ]; then
	if [ ! -f ${1} ] ; then
	    echo "ERROR : IGCM_sys_Put_Rest ${1} DOES NOT EXIST ."
	    IGCM_debug_Exit "IGCM_sys_Put_Rest"
	fi

	typeset RET
	#
	IGCM_sys_Chmod 444 ${1}
	#
	# Only if we use rsync
	#IGCM_sys_MkdirArchive $( dirname $2 )
	#
	#USUAL WAY
	mfput $1 $2 > out_rsync 2>&1
  	RET=$?

# 	#RSYNC WITH NETWORK RSH CALL
# 	echo ${RSYNC_FRONT} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RUN_DIR}/$1 ${RHOST}:${2} > out_rsync 2>&1
# 	${RSYNC_FRONT} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RUN_DIR}/$1 ${RHOST}:${2} >> out_rsync 2>&1

#  	#RSYNC WITH NFS USE
#  	echo ${RSYNC_FRONT} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} > out_rsync 2>&1
#  	${RSYNC_FRONT} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} >> out_rsync 2>&1
	
#  	RET=$?
#  	IGCM_sys_Rsync_out $RET

#  	${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#  	(( RET=RET+$? ))

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Put_Rest : error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Put_Rest"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Put_Rest"
}

#D-#==================================================
#D-function IGCM_sys_Put_Out
#D-* Purpose: Copy a file on $(ARCHIVE) after have chmod it in readonly
#D-* Examples:
#D-
function IGCM_sys_Put_Out {
    IGCM_debug_PushStack "IGCM_sys_Put_Out" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Put_Out :" $@
    fi
    if [ $DRYRUN = 0 ]; then
	if [ ! -f ${1} ] ; then
	    echo "WARNING : IGCM_sys_Put_Out ${1} DOES NOT EXIST ."
	    IGCM_debug_PopStack "IGCM_sys_Put_Out"
	    return 1
	fi

	typeset RET
	#
	IGCM_sys_Chmod 444 ${1}
	#
	# Only if we use rsync
	#IGCM_sys_MkdirArchive $( dirname $2 )
	#
	#USUAL WAY
	mfput $1 $2 > out_rsync 2>&1
	RET=$?

# 	#RSYNC WITH NETWORK RSH CALL
# 	echo ${RSYNC_FRONT} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RUN_DIR}/$1 ${RHOST}:${2} > out_rsync 2>&1
# 	${RSYNC_FRONT} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RUN_DIR}/$1 ${RHOST}:${2} >> out_rsync 2>&1

#  	#RSYNC WITH NFS USE
#  	echo ${RSYNC_FRONT} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} > out_rsync 2>&1
#  	${RSYNC_FRONT} ${RSYNC_opt} ${RUN_DIR}/$1 ${2} >> out_rsync 2>&1

#  	RET=$?
#  	IGCM_sys_Rsync_out $RET

#  	${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#  	(( RET=RET+$? ))

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Put_Out : error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Put_Out"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Put_Out"
    return 0
}

#D-#==================================================
#D-function IGCM_sys_Get
#D-* Purpose: Get a file from ${ARCHIVE}
#D-* Examples: IGCM_sys_Get myfile /destpath/myfile_with_PREFIX
#D-            IGCM_sys_Get /l Array_contain_myfiles /destpath/
function IGCM_sys_Get {
    IGCM_debug_PushStack "IGCM_sys_Get" $@

    typeset DEST RET dm_liste ifile target

    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Get :" $@
    fi
    if [ $DRYRUN -le 2 ]; then
	if [ X${1} = X'/l' ] ; then
            # test if the first file is present in the old computation :
	    eval set +A dm_liste \${${2}}
	else
	    dm_liste=${1}
	fi
	eval DEST=\${${#}}

        # test if the (first) file is present in the old computation :
	IGCM_sys_TestFileArchive ${dm_liste[0]}
	RET=$?
        if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Get, ERROR : regular file ${dm_liste[0]} DOES NOT EXIST ."
	    IGCM_debug_Exit "IGCM_sys_Get"
	    return
        fi

	# SD : dm_liste is not suited for computing job
	#      because we change filename during transfert
	#      dm_liste is better suited for post-treatment
	#dm_liste=" "
  	#(( ifile=1 ))
	#while [ $ifile -lt $# ] ; do
	#    dm_liste=$( eval echo ${dm_liste} " "\${${ifile}} )
	#    (( ifile = ifile + 1 ))
	#done
	#DEST=$( eval echo \${${#}} )

	#USUAL WAY
	mfget ${dm_liste[*]} ${DEST} > out_rsync 2>&1
	#mfget $1 $2 > out_rsync 2>&1
  	RET=$?

# 	#RSYNC WITH NETWORK RSH CALL
# 	echo ${RSYNC_FRONT} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RHOST}:"${dm_liste}" ${RHOST}:${RUN_DIR}/${DEST} > out_rsync 2>&1
# 	${RSYNC_FRONT} ${RSYNC_opt} --rsync-path=${REMOTE_RSYNC} -e rsh ${RHOST}:"${dm_liste}" ${RHOST}:${RUN_DIR}/${DEST} >> out_rsync 2>&1

#  	#RSYNC WITH NFS USE
#  	echo ${RSYNC_FRONT} ${RSYNC_opt} ${dm_liste} ${RUN_DIR}/${DEST} > out_rsync 2>&1
#  	${RSYNC_FRONT} ${RSYNC_opt} ${dm_liste} ${RUN_DIR}/${DEST} >> out_rsync 2>&1

# 	RET=$?
#  	IGCM_sys_Rsync_out $RET

#  	${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#  	(( RET=RET+$? ))

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Get : copy error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Get"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Get"
}

############################################################## A REVOIR !!

#D-#==================================================
#D-function IGCM_sys_Rapatrie
#D-* Purpose: Rapatrie
#D-* Examples:
#D-
function IGCM_sys_Rapatrie {
    IGCM_debug_PushStack "IGCM_sys_Rapatrie" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Rapatrie :" $@
    fi

    typeset RET=0

    IGCM_sys_Get ${R_STOCKAGE}/$2 $1 ; 
    let $(( RET=RET+$? ))
    IGCM_sys_Cd $1 ; 
    let $(( RET=RET+$? ))
    IGCM_sys_UnTar $2 ; 
    let $(( RET=RET+$? ))

    if [ ${RET} -gt 0 ] ; then
	echo "IGCM_sys_Rapatrie : erreur."
	IGCM_debug_Exit "IGCM_sys_Rapatrie"
    fi
    IGCM_debug_PopStack "IGCM_sys_Rapatrie"
}

############################################################## A FINIR !!

#D-#==================================================
#D-function IGCM_sys_GetDate_FichWork
#D-* Purpose: donne la date filesys d'un fichier sur le filesystem WORK
#D-* Examples:
#D-
function IGCM_sys_GetDate_FichWork {
    IGCM_debug_PushStack "IGCM_sys_FichWork" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_GetDate_FichWork :" $@
    fi
    # donne la date filesys d'un fichier sur la machine work
    IGCM_debug_PopStack "IGCM_sys_FichWork"
}

#D-#==================================================
#D-function IGCM_sys_GetDate_FichArchive
#D-* Purpose: donne la date filesys d'un fichier sur le filesystem ARCHIVE 
#D-* Examples:
#D-
function IGCM_sys_GetDate_FichArchive {
    IGCM_debug_PushStack "IGCM_sys_FichArchive" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_GetDate_FichArchive :" $@
    fi
    IGCM_debug_PopStack "IGCM_sys_FichArchive"
}

##############################################################
# REBUILD OPERATOR

function IGCM_sys_rebuild {
    IGCM_debug_PushStack "IGCM_sys_rebuild" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_rebuild :" $@
    fi
    /home/rech/psl/rpsl035/SX/bin/rebuild -f -o $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_rebuild : erreur ${@}."
       IGCM_debug_Exit "rebuild"
    fi

    IGCM_debug_PopStack "IGCM_sys_rebuild"
}

############################################################
# Activate Running Environnment Variables

function IGCM_sys_activ_variables {
    IGCM_debug_PushStack "IGCM_sys_activ_variables"
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_activ_variables "
    fi

# --------------------------------------------------------------------
#D- MPI specifications
# --------------------------------------------------------------------

#D-- MPISUSPEND
    export MPISUSPEND=ON
#D-- MPIPROGINF
    export MPIPROGINF=ALL

# --------------------------------------------------------------------
#D- Other specifications
# --------------------------------------------------------------------

#D- max number of character/line in output job
    export F_SYSLEN=5000
#D- number of error that can be admitted on the NEC
    export F_ERRCNT=0
#D- global performance
    export F_PROGINF=DETAIL
#D- activate ftrace (with -ftrace)
    export F_FTRACE=YES
#D- communication information (with -ftrace)
    export MPICOMMINF=ALL
#D- I/O performance (FORTRAN I/O only not netCDF)
    export F_FILEINF=NO
# netCDF I/O performance
    export NC_FILEINF=NO    

    IGCM_debug_PopStack "IGCM_sys_activ_variables"

}

############################################################
# Desactivate Running Environnment Variables

function IGCM_sys_desactiv_variables {
    IGCM_debug_PushStack "IGCM_sys_desactiv_variables"
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_desactiv_variables "
    fi
# --------------------------------------------------------------------
#D- MPI specifications
# --------------------------------------------------------------------

#D-- MPIPROGINF
    export MPIPROGINF=NO

# --------------------------------------------------------------------
#D- Other specifications
# --------------------------------------------------------------------

#D- global performance
    export F_PROGINF=NO  

    IGCM_debug_PopStack "IGCM_sys_desactiv_variables"
  
}

############################################################
# Build run file

function IGCM_sys_build_run_file {
    IGCM_debug_PushStack "IGCM_sys_build_run_file" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_build_run_file " $@
    fi
    (( NUM_PROC_ATM = BATCH_NUM_PROC_TOT - 1 ))
    (( NUM_PROC_OASIS = BATCH_NUM_PROC_TOT - NUM_PROC_ATM ))
    (( NUM_PROC_OCE = BATCH_NUM_PROC_TOT - NUM_PROC_ATM ))
    
    if [ $1 = MPI2 ]; then
	cat <<EOF > run_file
-p 1 -np 1 -e ./oasis
EOF
	(( NUM_PROCESS = BATCH_NUM_PROC_TOT + 1 ))
	config_UserChoices_JobRunOptions='"-max_np ${NUM_PROCESS} -f"'

    elif [ $1 = MPI1 ]; then
	cat <<EOF > run_file
-p $NUM_PROC_OASIS -e ./oasis
-p $NUM_PROC_ATM -e ./lmdz.x
-p $NUM_PROC_OCE -e ./opa.xx
EOF
 
    fi

    IGCM_debug_PopStack "IGCM_sys_build_run_file"
  
}
